﻿using System;
using System.ComponentModel.Composition.Hosting;
using System.IO;
using System.Windows;
using Microsoft.Practices.Prism.MefExtensions;
using Microsoft.Practices.Prism.Regions;
using Telerik.Windows.Controls;
using Microsoft.Practices.ServiceLocation;

namespace OutlookBarTest
{
    /// <summary>
    /// A bootstrapper for OutlookBarTest application managing the MEF container and module catalog.
    /// </summary>
    [CLSCompliant(false)]
    public class ShellBootstrapper : MefBootstrapper
    {
        /// <summary>
        /// A string containing the default path for modules folder.
        /// </summary>
        private string modulesPath = @"./Modules";

        /// <summary>
        /// Configures the <see cref="P:Microsoft.Practices.Prism.MefExtensions.MefBootstrapper.AggregateCatalog"/> used by MEF.
        /// </summary>
        /// <returns>
        /// An <see cref="P:Microsoft.Practices.Prism.MefExtensions.MefBootstrapper.AggregateCatalog"/> to be used by the bootstrapper.
        /// </returns>
        protected override AggregateCatalog CreateAggregateCatalog()
        {
            if (!Directory.Exists(this.modulesPath))
            {
                Directory.CreateDirectory(this.modulesPath);
            }

            return base.CreateAggregateCatalog();
        }

        /// <summary>
        /// Configures the <see cref="P:Microsoft.Practices.Prism.MefExtensions.MefBootstrapper.AggregateCatalog"/> used by MEF.
        /// </summary>
        protected override void ConfigureAggregateCatalog()
        {
            base.ConfigureAggregateCatalog();

            this.AggregateCatalog.Catalogs.Add(new AssemblyCatalog(typeof(ShellBootstrapper).Assembly));
            this.AggregateCatalog.Catalogs.Add(new DirectoryCatalog(this.modulesPath));
        }

        protected override RegionAdapterMappings ConfigureRegionAdapterMappings()
        {
            var mappings = base.ConfigureRegionAdapterMappings();
            var factory = this.Container.GetExportedValue<IRegionBehaviorFactory>();

            mappings.RegisterMapping(typeof(RadOutlookBar), new RadOutlookBarRegionAdapter(factory));

            return mappings;
        }

        /// <summary>
        /// Creates the shell or main window of the application.
        /// </summary>
        /// <returns>
        /// The shell of the application.
        /// </returns>
        protected override DependencyObject CreateShell()
        {
            DependencyObject shell = new Shell();

            return shell;
        }

        /// <summary>
        /// Initializes the shell.
        /// </summary>
        protected override void InitializeShell()
        {
            base.InitializeShell();

            Application.Current.MainWindow = (Window)this.Shell;
            Application.Current.MainWindow.Show();
        }
    }
}
